<?php
/**
 * Contabilizacao.
 *
 * @package Logic
 * @author  alessandrobartels
 *
 */
class Contabilizacao {

    private $_nrorg;
    private $_nrOrgPadrao;
    private $_simulacao;
    private $_mes_competencia;

    private $_dtInicio;
    private $_dtFim;
    private $_dtAux;

    private $_formulario;
    
    private $_evento;
    private $_itens_calculo;
    private $_totaliza_eventos;
    private $_resumo_contabil;
    private $_evento_contabil;
    private $_historicoPadrao;
    private $_contaContabil;

    private $_estruturacao;

    private $_movimentacao;


    /**
     * Construtor.
     *
     * @param integer    $nrorg           Numero da Organizacao.
     * @param integer    $nrOrgPadrao     Numero da Organizacao Padrao.
     * @param TkDateTime $mes_competencia Mes de competencia.
     * @param string     $simulacao       Tipo de processo.
     */
    public function __construct($nrorg, $nrOrgPadrao, TkDateTime $mes_competencia, $simulacao, $formulario){
        $this->_nrorg = $nrorg;
        $this->_nrOrgPadrao = $nrOrgPadrao;
        $this->_simulacao = $simulacao;
        $this->_mes_competencia = $mes_competencia;

        $this->_dtInicio = new TkDateTime($this->_mes_competencia);
        $this->_dtFim = new TkDateTime($this->_mes_competencia);
        $this->_dtFim->addMonth(1)->subDay(1);
        $this->_dtAux = new TkDateTime();

        $this->_evento = new FpaeventomEntity(null, $nrorg, $nrOrgPadrao, $mes_competencia);
        $this->_itens_calculo = new FpaitecalcfolhaEntity();
        $this->_resumo_contabil = new FparesumoctblEntity();
        $this->_evento_contabil = new FpaeventoctblEntity();
        $this->_historicoPadrao = new HistpadrEntity();
        $this->_contaContabil = new ContctblEntity();
        $this->_movimentacao = new GpemovimentacaoEntity();
        $this->_hierarquia_org = new HierarquiaorgEntity();

        $this->_estruturacao = new Estruturacao($this->_nrorg, $this->_nrOrgPadrao, $this->_mes_competencia);

        $this->_inicializaVariaveis($formulario);
    }


    private function _inicializaVariaveis($formulario){

        $this->_estruturacao->carregaHierarquias($formulario->getNrhierarqorg());
        $this->_estruturacao->carregaTipoHierarquia($formulario->getNrhierarqorg());
    }


    /**
     * Executa o processo de contabilizacao.
     *
     * @param GpevinculomEntity            $vinculo                      Instancia de GpevinculomEntity.
     * @param array                        $lista_calculo_folha          Array de numero de calculos de folha.
     * @param integer                      $nr_tipo_estrutura_rel        Numero da estrutura relacionada (filha).
     * @param HierarquiaorgEntity          $hierarquias                  Instancia de HierarquiaorgEntity.
     * @param DemonstrativosContabilizacao $demonstrativosContabilizacao Instancia de DemonstrativosContabilizacao.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    /*public function contabilizar(
        GpevinculomEntity $vinculo, array $lista_calculo_folha, $nr_tipo_estrutura_rel,
        HierarquiaorgEntity $hierarquias, DemonstrativosContabilizacao $demonstrativosContabilizacao
    ){*/
     public function contabilizar(
        GpevinculomEntity $vinculo, array $lista_calculo_folha, DemonstrativoContabilizacao $demonstrativosContabilizacao
    ){

        // Implementar o demonstrativo.
      /*  if ($demonstrativosContabilizacao === true) {
            true;
        }*/

     //  $nr_estrutura_sup = $this->_determinaEstruturaSuperior($vinculo, $nr_tipo_estrutura_rel, $hierarquias);
        $estruturaResp = $this->_estruturacao->retornaEstruturaResponsavel($vinculo->getNrvinculom(), $this->_dtInicio, $this->_dtFim);
        $nr_estrutura_sup = $estruturaResp->getNrestruturam();
        $this->_totaliza_eventos = array();

        // Totaliza o array de eventos do vinculo.
        $totalizou = $this->_totalizaResumoEventos($vinculo, $lista_calculo_folha);

        if ($totalizou === true) {

            // Ja retorna as lista de resumos contabeis
            $cont = $this->_resumo_contabil->criaListaResumoContabil(
                $this->_nrorg, $this->_mes_competencia, $nr_estrutura_sup
            );

            // Percorre array de eventos totalizados persistindo os eventos associados a estrutura.
            foreach ($this->_totaliza_eventos as $nr_evento => $valor_evento) {
                $data_rescisao_vinculo = $vinculo->getDtrescisaovinc();

                // Pesquisa contas contabeis.
                $evento_contabil = $this->_pesquisaContasContabeis(
                    $data_rescisao_vinculo, $nr_evento, $nr_estrutura_sup
                );

                // Se o evento nao estiver associado a contabilizacao, pega o proximo evento.
                if (is_object($evento_contabil) === false) {
                    continue;
                } else {
                    /* Pesquisa no resumo se ja existe conta de debito naquele mes de competencia e para aquela
                     * estrutura.
                     */
                    $this->_resumo_contabil->atualizaListaResumoContabil(
                        $this->_nrorg, $this->_mes_competencia, $nr_estrutura_sup,
                        FparesumoctblEntity::IDTIPOLANC_DEBITO, $evento_contabil->getCdcontctbldeb(),
                        $evento_contabil->getCdhistpadrdeb(), $valor_evento
                    );

                    /* Pesquisa no resumo se ja existe conta de credito naquele mes de competencia e para aquela
                     * estrutura.
                     */
                    $this->_resumo_contabil->atualizaListaResumoContabil(
                        $this->_nrorg, $this->_mes_competencia, $nr_estrutura_sup,
                        FparesumoctblEntity::IDTIPOLANC_CREDITO, $evento_contabil->getCdcontctblcre(),
                        $evento_contabil->getCdhistpadrcre(), $valor_evento
                    );
                }
            }


            if ($this->_simulacao === false) {
                $this->_resumo_contabil->persist();
                // Mostrar resultado na tela.
                // Retirar mensagem.
                // $msg = utf8_encode('Simulacao realizada.');
                //tk_warning($msg);
                //     return $demonstrativosContabilizacao->addEventoContabilizacao($this->_totaliza_eventos);

            } else {
            }

        } 

    }





    /**
     * Enter description here ...
     *
     * @param GpevinculomEntity   $vinculo               Instancia de GpevinculomEntity.
     * @param integer             $nr_tipo_estrutura_rel Numero da estrutura relacionada (filha).
     * @param HierarquiaorgEntity $hierarquias           Instancia de HierarquiaorgEntity.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function retrocedeResumoEventos(
        GpevinculomEntity $vinculo, $nr_tipo_estrutura_rel, $hierarquias
    ){
        // Retorna os itens do resumo.
//        $nr_estrutura = $this->_determinaEstruturaSuperior($vinculo, $nr_tipo_estrutura_rel, $hierarquias);
        $estruturaResp = $this->_estruturacao->retornaEstruturaResponsavel($vinculo->getNrvinculom(), $this->_dtInicio, $this->_dtFim);
        $nr_estrutura = $estruturaResp->getNrestruturam();
        $gerouLista = $this->_resumo_contabil->criaListaResumoContabil(
            $this->_nrorg, $this->_mes_competencia, $nr_estrutura
        );
        if ($gerouLista === true) {
            $this->_resumo_contabil->retrocedeListaResumoContabil();
            $this->_resumo_contabil->persist();
        }
    }

    /**
     * Retorna o numero da estrutura superior.
     *
     * @param GpevinculomEntity   $vinculo               Instancia de GpevinculomEntity.
     * @param integer             $nr_tipo_estrutura_rel Numero da estrutura relacionada (filha).
     * @param HierarquiaorgEntity $hierarquias           Instancia de HierarquiaorgEntity.
     *
     * @throws Exception Movimentacao inexistente.
     * @return integer
     */
    private function _determinaEstruturaSuperior(
        GpevinculomEntity $vinculo, $nr_tipo_estrutura_rel, $hierarquias
    ){
        // Metodo duplicado. Ir para hierarquia.
        $movimentacao_recente = $this->_movimentacao->retornaMovimentacaoRecentePorEstrutura(
            $this->_nrorg, $nr_tipo_estrutura_rel, $vinculo->getNrvinculom(),
            $this->_mes_competencia
        );

        if (is_object($movimentacao_recente) === false) {
            $msg = 'Movimentacao inexistente';
            throw new Exception($msg);
        }

        $hierarquia = $this->_hierarquia_org->procuraHierarquiaPorEstruturaRel(
            $movimentacao_recente->getNrestruturam(), $hierarquias
        );

        return $hierarquia->getNrestruturamsup();
    }


    /**
     * Enter description here ...
     *
     * @param GpevinculomEntity $vinculo             Instancia de GpevinculomEntity.
     * @param array             $lista_calculo_folha Lista com numero de calculo de folha.
     *
     * @return boolean
     *
     * @author alessandrobartels
     */
    private function _totalizaResumoEventos(GpevinculomEntity $vinculo, array $lista_calculo_folha){
        // Percorre os numeros de calculo totalizando os eventos.
        foreach ($lista_calculo_folha as $nr_calculo_folha) {
            $quantidade_itens = $this->_itens_calculo->retornaItensPorVinculo(
                $vinculo->getNrorg(), $vinculo->getNrvinculom(), $nr_calculo_folha
            );
            /* @todo parar o loop no calculo de folha ativo caso nao tenha itens (verificar) */
            if ($quantidade_itens === 0) {
                return false;
            }

            $itens = $this->_itens_calculo->getItems();

            // Loop itens de calculo.
            foreach ($itens as $item) {
                /*$evento = $this->_evento->retornaEvento($item->getNreventom());
                $posicao = $item->getNreventom().' - '.$evento->historico->getNmeventoh();*/
                @$this->_totaliza_eventos[$item->getNreventom()] += $item->getVreventofolha();
            }
        }

        return true;
    }


    /**
     * Enter description here ...
     *
     * @param string  $data_rescisao_vinculo Data de rescisao do vinculo.
     * @param integer $nr_evento             Numero do evento.
     * @param integer $nr_estrutura_sup      Numero da estrutura superior (pai).
     *
     * @return FpaeventoctblEntity
     *
     * @author alessandrobartels
     */
    private function _pesquisaContasContabeis($data_rescisao_vinculo, $nr_evento, $nr_estrutura_sup){
        if ($data_rescisao_vinculo === null) {
            $evento_contabil = $this->_evento_contabil->pesquisaContasContabeis(
                $this->_nrorg, $nr_evento, $nr_estrutura_sup,
                $this->_mes_competencia, FpaeventoctblEntity::IDVINCATIVO_NAO
            );
            if (is_object($evento_contabil) === false) {
                $evento_contabil = $this->_evento_contabil->pesquisaContasContabeis(
                    $this->_nrorg, $nr_evento, $nr_estrutura_sup,
                    $this->_mes_competencia, FpaeventoctblEntity::IDVINCATIVO_SIM
                );
            }
        } else {
            $evento_contabil = $this->_evento_contabil->pesquisaContasContabeis(
                $this->_nrorg, $nr_evento, $nr_estrutura_sup,
                $this->_mes_competencia, FpaeventoctblEntity::IDVINCATIVO_SIM
            );
        }

        return $evento_contabil;
    }

//    public function setTotalizaEventos($totaliza_eventos){
//        $this->_totaliza_eventos = $totaliza_eventos;
//    }

    public function getTotalizaEventos(){
        return $this->_totaliza_eventos;
    }

    public function getResumoContabil() {
        return $this->_resumo_contabil;
    }


    public function preparaDemonstrativo($demonstrativo){
        $itens = $this->_resumo_contabil->getItems();

        foreach($itens as $item){
            $dsContaContabil = $this->_contaContabil->getDescricaoContaContabil($this->_nrorg, $item->getCdcontctbl());
            $dsHistorico = $this->_historicoPadrao->getDescricaoHistorico($this->_nrorg, $item->getCdhistpadr());
            $demonstrativo->addRegistroContabilizacao(
                $item->getCdcontctbl(), $dsContaContabil, $item->getCdhistpadr(), $dsHistorico, $item->getVrlancamento(), $item->getIdtipolanc()
            );
        }

        return $demonstrativo;

    }


}